"""
=======================================================================
Frequency features
=======================================================================
'''Frequency domain features for numerical time series data'''
"""
from scipy import signal, interpolate
import numpy as np
import SWaN_accel.detect_peaks as detect_peaks
from SWaN_accel.utils import *

class FrequencyFeature:
    def __init__(self, X, sr, freq_range=None):
        self._X = X
        self._sr = sr
        self._freq_range = freq_range

    def fft(self):
        # if(len(self._X)==0):
        #     return
        freq, time, Sxx = signal.spectrogram(
            self._X,
            fs=self._sr,
            window='hamming',
            nperseg=self._X.shape[0],
            noverlap=0,
            detrend='constant',
            return_onesided=True,
            scaling='density',
            axis=0,
            mode='magnitude')
        # print(time,len(freq))
        # interpolate to get values in the freq_range
        if self._freq_range is not None:
            self._freq = interpolate(freq, Sxx)
            Sxx_interpolated = interpolate_f(freq_range)
        else:
            self._freq = freq
            Sxx_interpolated = Sxx
        Sxx_interpolated = np.squeeze(Sxx_interpolated)
        self._Sxx = vec2colarr(Sxx_interpolated)
        self._Sxx = np.abs(self._Sxx)
        return self

    def dominant_frequency(self, n=1):
        if hasattr(self, '_freq_peaks'):
            result = list(
                map(
                    lambda i: self._freq_peaks[i][n -
                                                  1] if
                    len(self._freq_peaks[i]) >= n else -1,
                    range(0, self._Sxx.shape[1])))
            result = vec2rowarr(np.array(result))
#             result = add_name(
#                 result, self.dominant_frequency.__name__ + '_' + str(n))
            result = add_name(
                result, ['X_DOMFREQ','Y_DOMFREQ','Z_DOMFREQ'])
            return result
        else:
            raise ValueError('Please run spectrogram and peaks methods first')

#     def dominant_frequency_x(self, n=1):
#         if hasattr(self, '_freq_peaks'):
#             result = list(
#                 map(
#                     lambda i: self._freq_peaks[i][n -
#                                                   1] if
#                     len(self._freq_peaks[i]) >= n else -1,
#                     range(0, self._Sxx.shape[1])))
#             result = vec2rowarr(np.array(result[0))
#             result = add_name(
#                 result, self.X_DOMFREQ.__name__)
#             return result
#         else:
#             raise ValueError('Please run spectrogram and peaks methods first')
                                                
    
                                                
    def dominant_frequency_power(self, n=1):
        if hasattr(self, '_Sxx_peaks'):
            result = list(
                map(
                    lambda i: self._Sxx_peaks[i][n -
                                                 1] if
                    len(self._Sxx_peaks[i]) >= n else -1,
                    range(0, self._Sxx.shape[1])))
            result = vec2rowarr(np.array(result))
            result = add_name(
                result, ['X_DOMFREQ_POWER','Y_DOMFREQ_POWER','Z_DOMFREQ_POWER'])
            return result
        else:
            raise ValueError('Please run spectrogram and peaks methods first')
                                                
#     def dominant_frequency_power_x(self, n=1):
#         if hasattr(self, '_Sxx_peaks'):
#             result = list(
#                 map(
#                     lambda i: self._Sxx_peaks[i][n -
#                                                  1] if
#                     len(self._Sxx_peaks[i]) >= n else -1,
#                     range(0, self._Sxx.shape[1])))
#             result = vec2rowarr(np.array(result[0))
#             result = add_name(
#                 result, self.X_DOMFREQ_POWER.__name__)
#             return result
#         else:
#             raise ValueError('Please run spectrogram and peaks methods first')
                                                
    def total_power(self):
        if hasattr(self, '_Sxx'):
            result = vec2rowarr(np.sum(self._Sxx, axis=0))
            result = add_name(result, ['X_TOTPOW','Y_TOTPOW','Z_TOTPOW'])
            return result
        else:
            raise ValueError('Please run spectrogram first')
                                                
#     def total_power_X(self):
#         if hasattr(self, '_Sxx'):
#             result = vec2rowarr(np.sum(self._Sxx, axis=0))
#             result = add_name(result, self.X_TOTPOW.__name__)
#             return result
#         else:
#             raise ValueError('Please run spectrogram first')

    def limited_band_dominant_frequency(self, low=0, high=np.inf, n=1):
        def _limited_band_df(i):
            freq = self._freq_peaks[i]
            indices = (freq >= low) & (freq <= high)
            limited_freq = freq[indices]
            if len(limited_freq) < n:
                return -1
            else:
                return limited_freq[n-1]
        if not hasattr(self, '_freq_peaks'):
            raise ValueError('Please run spectrogram and peaks methods first')

        result = list(
            map(_limited_band_df,
                range(0, self._Sxx.shape[1])))

        result = vec2rowarr(np.array(result))
        result = add_name(
            result, self.limited_band_dominant_frequency.__name__ + '_' + str(n))
        return result

    def limited_band_dominant_frequency_power(self, low=0, high=np.inf, n=1):
        def _limited_band_df_power(i):
            freq = self._freq_peaks[i]
            Sxx = self._Sxx_peaks[i]
            indices = (freq >= low) & (freq <= high)
            limited_Sxx = Sxx[indices]
            if len(limited_Sxx) < n:
                return -1
            else:
                return limited_Sxx[n-1]
        if not hasattr(self, '_freq_peaks'):
            raise ValueError('Please run spectrogram and peaks methods first')

        result = list(
            map(_limited_band_df_power,
                range(0, self._Sxx.shape[1])))

        result = vec2rowarr(np.array(result))
        result = add_name(
            result, self.limited_band_dominant_frequency_power.__name__ + '_' + str(n))
        return result

    def limited_band_total_power(self, low=0, high=np.inf):
        if not hasattr(self, '_freq'):
            raise ValueError('Please run spectrogram first')
        indices = (self._freq >= low) & (self._freq <= high)
        limited_Sxx = self._Sxx[indices, :]
        limited_total_power = vec2rowarr(np.sum(limited_Sxx, axis=0))
        # limited_total_power = add_name(
        #     limited_total_power, self.limited_band_total_power.__name__)
        limited_total_power = add_name(
            limited_total_power, ['X_total_power_between_'+str(low)+'_'+str(high),'Y_total_power_between_'+str(low)+'_'+str(high),'Z_total_power_between_'+str(low)+'_'+str(high)])
        return limited_total_power

    def highend_power(self):
        if hasattr(self, '_Sxx'):
            result = self.limited_band_total_power(low=3.5)
            result = add_name(
                result.values, self.highend_power.__name__)
            return result
        else:
            raise ValueError('Please run spectrogram first')

    def dominant_frequency_power_ratio(self, n=1):
        power = self.total_power().values
        result = np.divide(self.dominant_frequency_power(n=n).values,
                           power, out=np.zeros_like(power), where=power != 0)
        result = add_name(
            result, self.dominant_frequency_power_ratio.__name__ + '_' + str(n))
        return result

    def middlerange_dominant_frequency(self):
        result = self.limited_band_dominant_frequency(low=0.6, high=2.6, n=1)
        result = add_name(
            result.values, self.middlerange_dominant_frequency.__name__)
        return result

    def middlerange_dominant_frequency_power(self):
        result = self.limited_band_dominant_frequency_power(low=0.6, high=2.6,
                                                            n=1)
        result = add_name(
            result.values, self.middlerange_dominant_frequency_power.__name__)
        return result

    def peaks(self):
        def _sort_peaks(i, j):
            if len(i) == 0:
                sorted_freq_peaks = np.array([0])
                sorted_Sxx_peaks = np.array([np.nanmean(self._Sxx, axis=0)[j]])
            else:
                freq_peaks = self._freq[i]
                Sxx_peaks = self._Sxx[i, j]
                sorted_i = np.argsort(Sxx_peaks)
                sorted_i = sorted_i[::-1]
                sorted_freq_peaks = freq_peaks[sorted_i]
                sorted_Sxx_peaks = Sxx_peaks[sorted_i]
            return (sorted_freq_peaks, sorted_Sxx_peaks)

        n_axis = self._Sxx.shape[1]
        m_freq = self._Sxx.shape[0]
        # at least 0.1 Hz different when looking for peak
        # if(len(self._freq)<10):
        #     print(self._freq)
        mpd = int(np.ceil(1.0 / (self._freq[1] - self._freq[0]) * 0.1))
        # print(self._Sxx.shape)
        # i = list(map(lambda x: detect_peaks.detect_peaks(
        #     x, mph=1e-3, mpd=mpd), list(self._Sxx.T)))
        #
        i = list(map(lambda x: detect_peaks.detect_peaks(
            x, mph=None, mpd=mpd), list(self._Sxx.T)))
        j = range(0, n_axis)
        result = list(map(_sort_peaks, i, j))
        self._freq_peaks = list(map(lambda x: x[0], result))
        self._Sxx_peaks = list(map(lambda x: x[1], result))
        return self



import SWaN_accel.spectrum as spectrum
import SWaN_accel.orientation as orientation
import pandas as pd
import SWaN_accel.config as config
import SWaN_accel.utils as utils
import SWaN_accel.energy as energy
from numpy.linalg import norm

def compute_extra_features(X,sampling):
   
    feature_list = []

    spectrum_feature_extractor = spectrum.FrequencyFeature(X, sr=sampling)
    spectrum_feature_extractor.fft()
    spectrum_feature_extractor.peaks()
    feature_list.append(spectrum_feature_extractor.dominant_frequency())
    feature_list.append(spectrum_feature_extractor.dominant_frequency_power())
    feature_list.append(spectrum_feature_extractor.total_power())


    ori_feature_extractor = orientation.OrientationFeature(X, subwins=config.winSize)
    ori_feature_extractor.estimate_orientation(unit='deg')
    feature_list.append(ori_feature_extractor.ori_x_median())
    feature_list.append(ori_feature_extractor.ori_y_median())
    feature_list.append(ori_feature_extractor.ori_z_median())
    feature_list.append(ori_feature_extractor.ori_var_sum())
    feature_list.append(ori_feature_extractor.ori_range_max())

    X_vm = utils.vec2colarr(norm(X, ord=2, axis=1))
    energy_feature_extractor = energy.EnergyFeature(X_vm, subwins=30)
    energy_feature_extractor.get_energies()
    feature_list.append(energy_feature_extractor.smv_energy_sum())
    feature_list.append(energy_feature_extractor.smv_energy_var())

    result = pd.concat(feature_list, axis=1)
    return result



import os, sys

root_folder = os.path.realpath(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..'))

def relpath(*dirs):
    return os.path.relpath(os.path.join(root_folder, *dirs), os.getcwd())

sys.path.insert(0, os.path.join(root_folder, 'scripts'))



import numpy as np
import pandas as pd

def as_float64(X):
    return X.astype(np.float64)

def add_name(X, name):
#     n_dim = X.shape[1]
#     names = [name.upper() + '_' + str(n) for n in range(0, n_dim)]
    if(isinstance(name,list)):
       names = [x.upper() for x in name]
    else:
       names = [name.upper()]
       
    result = pd.DataFrame(data=X, columns=names)
    return result

def vec2colarr(X):
    if len(X.shape) == 1:
        return np.reshape(X, (X.shape[0], 1))
    elif len(X.shape) == 2:
        return X
    elif len(X.shape) > 2:
        raise NotImplementedError('''vec2colarr only accepts 1D or 2D numpy
         array''')

def vec2rowarr(X):
    if len(X.shape) == 1:
        return np.reshape(X, (1, X.shape[0]))
    elif len(X.shape) == 2:
        return X
    elif len(X.shape) > 2:
        raise NotImplementedError('''vec2rowarr only accepts 1D or 2D numpy
         array''')
    else:
        return np.nan

def has_enough_samples(X, threshold=1):
    if len(X.shape) == 1:
        X = vec2colarr(X)
    return X.shape[0] >= threshold

def apply_over_subwins(X, func, subwins, **kwargs):
    win_length = int(np.floor(X.shape[0] / subwins))
    start_index = np.ceil((X.shape[0] % subwins) / 2)
    result = []
    if(win_length <1):
        # print("Data does not have points => number of sub windows")
        result.append(func(X, **kwargs))
    else:
        # print("##########################################################")
        # print(X.shape,win_length,start_index,subwins)
        # print(X[0,:])

        for i in range(0, subwins):
            indices = int(start_index) + np.array(range(
                i * win_length,
                (i + 1) * win_length
            ))
            subwin_X = X[indices, :]
            result.append(func(subwin_X, **kwargs))
    return result



import calendar, json, os, sys, time
from SWaN_accel import _root



def _get_dataset_folder():
    if os.path.exists(_root.relpath('datasets_custom')):
        return 'datasets_custom'
    return 'datasets'

def _get_labels_folder():
    if os.path.exists(_root.relpath('labels_custom')):
        return 'labels_custom'
    return 'labels'

def datasetDir(dataset):
    return _root.relpath(_get_dataset_folder(), dataset)

def datasetTileDir(dataset):
    return _root.relpath(_get_dataset_folder(), dataset, 'tiles')

def datasetConfigFilename(dataset):
    return _root.relpath(_get_dataset_folder(), dataset, 'config.json')

def latestLabelsFilename(dataset, session):
    return _root.relpath(_get_labels_folder(), dataset, session, 'labels.latest.json')

def logLabelsFilename(dataset, session):
    return _root.relpath(_get_labels_folder(), dataset, session, 'labels.log.jsons')

def exportFilename(dataset):
    return _root.relpath('export', dataset + '.csv')



def getLabelsSessions(dataset):
    sessions = []

    folder = _root.relpath(_get_labels_folder(), dataset)
    if os.path.exists(folder):
        for fn in os.listdir(folder):
            sessions.append(fn)

    return sessions

def getLabelsLatest(dataset):
    sessions = getLabelsSessions(dataset)

    labelsall = []
    for session in sessions:
        labelfilename = latestLabelsFilename(dataset, session)
        if os.path.exists(labelfilename):
            with open(labelfilename, 'rb') as lfile:
                labels = json.loads(lfile.read())
                labelsall.append(labels)
    return labelsall



def ensureDirExists(name, isFile):
    if isFile:
        dirname = os.path.dirname(name)
    else:
        dirname = name
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    return name



def activityJSON(a, wasprev):
    comma = ''
    if wasprev:
        comma = ','
    return comma + ('{"lo":%d, "hi":%d, "label":"%s"}' % (a[0], a[1], a[2]))



def timeMillisecondToTimeString(ms):
    sec = ms / 1000
    msec = ms % 1000

    tm = time.gmtime(sec)
    return time.strftime('%Y-%m-%d %H:%M:%S', tm) + ('.%03d' % msec)

def timeStringToTimeMillisecond(tm):
    parts = tm.split('.')
    if len(parts) == 1:
        sec = parts[0]
        msec = '000'
    elif len(parts) == 2:
        sec = parts[0]
        msec = parts[1]
        if len(msec) > 3:
            print('*** ERROR: time more detailed than milliseconds: ' + tm)
            sys.exit(-1)
        while len(msec) < 3:
            msec = msec + '0'
    else:
        print('*** ERROR: invalid time format: ' + tm)
        sys.exit(-1)

    parsed = None
    for fmt in ['%Y-%m-%d %H:%M:%S', '%m/%d/%Y %H:%M:%S']:
        try:
            parsed = time.strptime(sec, fmt)
            break
        except:
            pass

    if not parsed:
        print('*** ERROR: unparseable time format: ' + tm)
        sys.exit(-1)

    return 1000 * int(calendar.timegm(parsed)) + int(msec)



import pickle, datetime
import pandas as pd
import numpy as np

from SWaN_accel import config
from SWaN_accel import utils
from SWaN_accel import feature_set
pd.options.mode.chained_assignment = None  # default='warn'


col = ["HEADER_TIME_STAMP","X_ACCELERATION_METERS_PER_SECOND_SQUARED",
       "Y_ACCELERATION_METERS_PER_SECOND_SQUARED","Z_ACCELERATION_METERS_PER_SECOND_SQUARED"]

def get_feature_sleep(tdf, sampling):
    X_axes = utils.as_float64(tdf.values[:, 1:])
    result_axes = feature_set.compute_extra_features(X_axes, sampling)
    return result_axes


def main(df=None, file_path=None,sampling_rate=None):

    if(df is None) or (file_path is None) or (sampling_rate is None):
        print("One or all input arguments missing.")
        return

    try:
        import importlib.resources as pkg_resources
    except ImportError:
        # Try backported to PY<37 `importlib_resources`.
        import importlib_resources as pkg_resources

    # trainedModel = pickle.load(open(config.modelPath, "rb"))
    # standardScalar = pickle.load(open(config.scalePath, "rb"))

    trainedModel = pickle.load(pkg_resources.open_binary(__package__,config.modelPath))
    standardScalar = pickle.load(pkg_resources.open_binary(__package__,config.scalePath))

    time_grouper = pd.Grouper(key='HEADER_TIME_STAMP', freq='30s')
    grouped_df = df.groupby(time_grouper)

    print("Computing features...")
    feature_df = pd.DataFrame()
    for name, group in grouped_df:
        if len(group) > sampling_rate * 15:
            op = get_feature_sleep(group, sampling_rate)
            op['HEADER_TIME_STAMP'] = name
            feature_df = pd.concat([feature_df, op], ignore_index=True)

    final_feature_df = feature_df.dropna(how='any', axis=0, inplace=False)
    if final_feature_df.empty:
        print("No feature row computed or remaining after dropping zero rows. So not moving to prediction.")
        return

    final_feature_df.rename(columns={'HEADER_TIME_STAMP': 'START_TIME'}, inplace=True)
    final_feature_df['HEADER_TIME_STAMP'] = final_feature_df['START_TIME']
    final_feature_df['STOP_TIME'] = final_feature_df['START_TIME'] + pd.Timedelta(seconds=30)

    print(datetime.datetime.now().strftime("%H:%M:%S") + " Performing window-level classification...")
    final_feature_df = final_feature_df.dropna()
    subfdata = final_feature_df[config.feature_lis]
    sfdata = standardScalar.transform(subfdata)
    prediction_prob = trainedModel.predict_proba(sfdata)
    prediction = np.argmax(prediction_prob, axis=1)
    p = prediction.reshape((-1, 1))
    final_feature_df["PREDICTED"] = p
    final_feature_df['PROB_WEAR'] = prediction_prob[:, 0]
    final_feature_df['PROB_SLEEP'] = prediction_prob[:, 1]
    final_feature_df['PROB_NWEAR'] = prediction_prob[:, 2]

    final_feature_df.to_csv(file_path, index=False, float_format="%.3f", compression='infer')
    print("Created prediction file:" + file_path)

    return

# if __name__ == "__main__":
#     main()



from setuptools import setup

setup(
    name='SWaN_pack',
    version='1.0',
    packages=[''],
    url='',
    license='',
    author='binodtc',
    author_email='',
    description='Classification of 24-hour accelerometer dataset into wake-wear, sleep-wear, and non-wear'
)




"""

Computing features about accelerometer orientations

Author: Binod Thapa Chhetry

Date: Jul 10, 2018
"""
import numpy as np
from numpy.linalg import norm
from SWaN_accel.utils import *

class EnergyFeature:
    def __init__(self, X, subwins=30):
        self._X = X
        self._subwins = subwins

    @staticmethod
    def energies(X):
        X = as_float64(X)
        if not has_enough_samples(X):
            print(
                '''One of sub windows do not have enough samples, will ignore in
                feature computation''')
            energies = np.array([np.nan])
        else:
            energies = np.array([np.sum(np.square(X))/(X.shape[0])])
        return vec2rowarr(energies)
    
    
    def get_energies(self):
        result = apply_over_subwins(
           self._X, EnergyFeature.energies, subwins=self._subwins)

        self._energies = np.concatenate(result, axis=0)
        return self

    def smv_energy_sum(self):
        smv_energy_sum = np.nansum(self._energies, axis=0)
        result = vec2rowarr(smv_energy_sum)
        result = add_name(result, self.smv_energy_sum.__name__)
        return result

    def smv_energy_var(self):
        smv_energy_var = np.nanvar(self._energies, axis=0)
        result = vec2rowarr(smv_energy_var)
        result = add_name(result, self.smv_energy_var.__name__)
        return result

    